/*============================================================================
  Summary:  Demonstrates Dead Locking
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Create an Extended Event session
CREATE EVENT SESSION Deadlocks ON SERVER 
ADD EVENT sqlserver.lock_deadlock
(
    ACTION
	(
		sqlserver.database_id,
		sqlserver.plan_handle,
		sqlserver.session_id,
		sqlserver.sql_text
	)
),
ADD EVENT sqlserver.xml_deadlock_report
(
    ACTION
	(
		sqlserver.database_id,
		sqlserver.plan_handle,
		sqlserver.session_id,
		sqlserver.sql_text
	)
) 
ADD TARGET package0.ring_buffer
GO

-- Start the Event Session
ALTER EVENT SESSION Deadlocks ON SERVER
STATE = START
GO

USE TempDB
GO

CREATE TABLE T1
(
	C1 INT
)
GO

INSERT INTO T1 VALUES (1)
GO

CREATE TABLE T2
(
	C1 INT
)

INSERT INTO T2 VALUES (1)
GO

SELECT * FROM T1
SELECT * FROM T2
GO

BEGIN TRANSACTION

UPDATE T1 SET C1 = 2

UPDATE T2 SET C1 = 2

-- COMMIT TRANSACTION

DROP TABLE T1
DROP TABLE T2
GO

-- Analyze the collected information
SELECT CAST(target_data AS XML) FROM sys.dm_xe_session_targets t
INNER JOIN sys.dm_xe_sessions s ON s.[address] = t.event_session_address
WHERE s.name = 'Deadlocks'
GO

-- Drop the Event Session
DROP EVENT SESSION Deadlocks ON SERVER
GO