/*============================================================================
  Summary:  Demonstrates Bookmark Lookup Deadlocks
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE master
GO

CREATE DATABASE BookmarkLookupDL
GO

USE BookmarkLookupDL
GO

CREATE TABLE Deadlock
(
	Col1 INT NOT NULL PRIMARY KEY CLUSTERED,
	Col2 INT NOT NULL,
	Col3 INT NOT NULL
)
GO

CREATE NONCLUSTERED INDEX idx_Col3 ON Deadlock(Col3)
GO

INSERT INTO Deadlock VALUES (1, 1, 1)
GO

SELECT * FROM Deadlock
GO

SET TRANSACTION ISOLATION LEVEL REPEATABLE READ
GO

WHILE (1 = 1)
BEGIN
	UPDATE Deadlock
	SET Col1 = Col1 + 1
	WHERE Col3 = 1
END
GO

-- ===================================================
-- Demonstrates how to resolve this specific deadlock
-- ===================================================

-- Creates a Covering Non-Clustered Index
CREATE NONCLUSTERED INDEX idx_Col3 ON Deadlock(Col3)
INCLUDE (Col1, Col2)
WITH (DROP_EXISTING = ON)
GO

-- Set the database to RCSI (Read Committed Snapshot Isolation)
ALTER DATABASE BookmarkLookupDL SET READ_COMMITTED_SNAPSHOT ON
GO

-- Check if RCSI is now enabled
SELECT
	name,
	is_read_committed_snapshot_on
FROM sys.databases
WHERE database_id = DB_ID('BookmarkLookupDL')
GO

-- Disable Read Committed Snapshot Isolation (RCSI)
ALTER DATABASE BookmarkLookupDL SET READ_COMMITTED_SNAPSHOT OFF
GO

-- Check if RCSI is now disabled
SELECT
	name,
	is_read_committed_snapshot_on
FROM sys.databases
WHERE database_id = DB_ID('BookmarkLookupDL')
GO

-- Clean up
USE master
GO

DROP DATABASE BookmarkLookupDL
GO