/*============================================================================
  Summary:  Demonstrates Dead Locking because of a Missing Index
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- =========
-- Session 1
-- =========
USE master
GO

-- Create a new database
CREATE DATABASE DeadlockingDemo
GO

-- Use it
USE DeadlockingDemo
GO

-- Create a table without any indexes
CREATE TABLE Table1
(
	Column1 INT,
	Column2 INT
)
GO

-- Insert a few record
INSERT INTO Table1 VALUES (1, 1)
INSERT INTO Table1 VALUES (2, 2)
INSERT INTO Table1 VALUES (3, 3)
INSERT INTO Table1 VALUES (4, 4)
GO

-- Create a table without any indexes
CREATE TABLE Table2
(
	Column1 INT,
	Column2 INT
)
GO

-- Insert a few record
INSERT INTO Table2 VALUES (1, 1)
INSERT INTO Table2 VALUES (2, 2)
INSERT INTO Table2 VALUES (3, 3)
INSERT INTO Table2 VALUES (4, 4)
GO

BEGIN TRANSACTION
	-- Acquires an Exclusive Lock on the row
	UPDATE Table1 SET Column1 = 3 WHERE Column2 = 1

	-- Execute the query from Session 2...
	-- ...

	-- This query now requests a Shared Lock, but get's blocked, because the other session/transaction has an Exclusive Lock on one row, that is currently updated
	SELECT Column1 FROM Table2
	--WITH (INDEX = idx_Column2) -- The index hint is necessary, because in this tiny table, SQL Server will just scan the whole table...
	WHERE Column2 = 3

ROLLBACK TRANSACTION
GO

-- Create new indexes so that SQL Server has alternative access paths to the data
-- The previous 2 SELECT statements can be now done through the Nonclustered Index without acquiring a Shared Lock
-- on the table itself (which currently holds an Exclusive Lock from the UPDATE statement).
CREATE NONCLUSTERED INDEX idx_Column2 ON Table1(Column2)
CREATE NONCLUSTERED INDEX idx_Column2 ON Table2(Column2)
GO

-- Clean up
USE master
GO

DROP DATABASE DeadlockingDemo
GO