/*============================================================================
  Summary:  Demonstrates Deadlocks in Repeatable Read
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE AdventureWorks2014
GO

SET TRANSACTION ISOLATION LEVEL REPEATABLE READ
GO

-- =========================================================
-- Causes a deadlock when we access data in the wrong order
-- =========================================================
BEGIN TRANSACTION

	-- 2nd range of data
	SELECT * FROM Person.Person
	WHERE ModifiedDate = '20120209'

	UPDATE Person.Person
	SET FirstName = '...'
	WHERE ModifiedDate = '20120209'

	-- Switch to session 1
	-- ...

	-- 1st range of data
	-- This statement will cause a deadlock!
	SELECT * FROM Person.Person
	WHERE ModifiedDate = '20120208'

	UPDATE Person.Person
	SET FirstName = '...'
	WHERE ModifiedDate = '20120208'

ROLLBACK
GO

-- =========================================================
-- Causes a deadlock when we access data in the wrong order
-- =========================================================
BEGIN TRANSACTION

	SELECT * FROM Person.Person
	WHERE ModifiedDate = '20120208'

	-- This statement will cause a deadlock!
	UPDATE Person.Person
	SET FirstName = '...'
	WHERE ModifiedDate = '20120208'

	-- Switch to session 1
	-- ...

ROLLBACK
GO